@@ -21,8 +21,6 @@
  *
  */
 
-/* gcc -DHAVE_LIBNOTIFY -DTEST -Wall `pkg-config --cflags --libs gobject-2.0 gio-unix-2.0 glib-2.0 gtk+-2.0 libnotify` -o gsd-disk-space-test gsd-disk-space.c */
-
 #include "config.h"
 
 #include <sys/statvfs.h>
@@ -35,7 +33,7 @@
 #include <gio/gunixmounts.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
+#include <libnotify/notify.h>
 
 #include "gsd-disk-space.h"
 #include "gsd-ldsm-dialog.h"
@@ -48,12 +46,12 @@
 
 #define DISK_SPACE_ANALYZER        "baobab"
 
-#define GCONF_HOUSEKEEPING_DIR     "/apps/gnome_settings_daemon/plugins/housekeeping"
-#define GCONF_FREE_PC_NOTIFY_KEY   "free_percent_notify"
-#define GCONF_FREE_PC_NOTIFY_AGAIN_KEY "free_percent_notify_again"
-#define GCONF_FREE_SIZE_NO_NOTIFY  "free_size_gb_no_notify"
-#define GCONF_MIN_NOTIFY_PERIOD    "min_notify_period"
-#define GCONF_IGNORE_PATHS         "ignore_paths"
+#define SETTINGS_HOUSEKEEPING_DIR     "org.gnome.settings-daemon.plugins.housekeeping"
+#define SETTINGS_FREE_PC_NOTIFY_KEY   "free-percent-notify"
+#define SETTINGS_FREE_PC_NOTIFY_AGAIN_KEY "free-percent-notify-again"
+#define SETTINGS_FREE_SIZE_NO_NOTIFY  "free-size-gb-no-notify"
+#define SETTINGS_MIN_NOTIFY_PERIOD    "min-notify-period"
+#define SETTINGS_IGNORE_PATHS         "ignore-paths"
 
 typedef struct
 {
@@ -70,9 +68,10 @@ static double             free_percent_notify_again = 0.01;
 static unsigned int       free_size_gb_no_notify = 2;
 static unsigned int       min_notify_period = 10;
 static GSList            *ignore_paths = NULL;
-static unsigned int       gconf_notify_id;
-static GConfClient       *client = NULL;
+static GSettings         *settings = NULL;
 static GsdLdsmDialog     *dialog = NULL;
+static NotifyNotification *notification = NULL;
+
 static guint64           *time_read;
 
 static gchar*
@@ -170,6 +169,71 @@ ldsm_analyze_path (const gchar *path)
                         NULL, NULL, NULL, NULL);
 }
 
+static gboolean
+server_has_actions (void)
+{
+        gboolean has;
+        GList   *caps;
+        GList   *l;
+
+        caps = notify_get_server_caps ();
+        if (caps == NULL) {
+                fprintf (stderr, "Failed to receive server caps.\n");
+                return FALSE;
+        }
+
+        l = g_list_find_custom (caps, "actions", (GCompareFunc)strcmp);
+        has = l != NULL;
+
+        g_list_foreach (caps, (GFunc) g_free, NULL);
+        g_list_free (caps);
+
+        return has;
+}
+
+static void
+ignore_callback (NotifyNotification *n,
+                 const char         *action)
+{
+        g_assert (action != NULL);
+        g_assert (strcmp (action, "ignore") == 0);
+
+        /* Do nothing */
+
+        notify_notification_close (n, NULL);
+}
+
+static void
+examine_callback (NotifyNotification *n,
+                  const char         *action,
+                  const char         *path)
+{
+        g_assert (action != NULL);
+        g_assert (strcmp (action, "examine") == 0);
+
+        ldsm_analyze_path (path);
+
+        notify_notification_close (n, NULL);
+}
+
+static void
+empty_trash_callback (NotifyNotification *n,
+                      const char         *action)
+{
+        g_assert (action != NULL);
+        g_assert (strcmp (action, "empty-trash") == 0);
+
+        gsd_ldsm_trash_empty ();
+
+        notify_notification_close (n, NULL);
+}
+
+static void
+on_notification_closed (NotifyNotification *notification)
+{
+        notification = NULL;
+}
+
 static gboolean
 ldsm_notify_for_mount (LdsmMountInfo *mount,
                        gboolean       multiple_volumes,
@@ -181,56 +245,129 @@ ldsm_notify_for_mount (LdsmMountInfo *mount,
         gboolean has_trash;
         gboolean has_disk_analyzer;
         gboolean retval = TRUE;
-        const gchar *path;
+        gchar *path;
 
-        /* Don't show a dialog if one is already displayed */
-        if (dialog)
+        /* Don't show a notice if one is already displayed */
+        if (dialog != NULL || notification != NULL)
                 return retval;
 
         name = g_unix_mount_guess_name (mount->mount);
         free_space = (gint64) mount->buf.f_frsize * (gint64) mount->buf.f_bavail;
         has_trash = ldsm_mount_has_trash (mount);
-        path = g_unix_mount_get_mount_path (mount->mount);
+        path = g_strdup (g_unix_mount_get_mount_path (mount->mount));
 
         program = g_find_program_in_path (DISK_SPACE_ANALYZER);
         has_disk_analyzer = (program != NULL);
         g_free (program);
 
-        dialog = gsd_ldsm_dialog_new (other_usable_volumes,
-                                      multiple_volumes,
-                                      has_disk_analyzer,
-                                      has_trash,
-                                      free_space,
-                                      name,
-                                      path);
+        if (server_has_actions ()) {
+                char *free_space_str;
+                char *summary;
+                char *body;
+
+                free_space_str = g_format_size_for_display (free_space);
+
+                if (multiple_volumes) {
+                        summary = g_strdup_printf (_("Low Disk Space on \"%s\""), name);
+                        if (has_trash) {
+                                body = g_strdup_printf (_("The volume \"%s\" has only %s disk space remaining.  You may free up some space by emptying the trash."),
+                                                        name,
+                                                        free_space_str);
+                        } else {
+                                body = g_strdup_printf (_("The volume \"%s\" has only %s disk space remaining."),
+                                                        name,
+                                                        free_space_str);
+                        }
+                } else {
+                        summary = g_strdup (_("Low Disk Space"));
+                        if (has_trash) {
+                                body = g_strdup_printf (_("This computer has only %s disk space remaining.  You may free up some space by emptying the trash."),
+                                                        free_space_str);
+                        } else {
+                                body = g_strdup_printf (_("This computer has only %s disk space remaining."),
+                                                        free_space_str);
+                        }
+                }
+                g_free (free_space_str);
+
+                notification = notify_notification_new (summary, body, "drive-harddisk-symbolic");
+                g_free (summary);
+                g_free (body);
+
+                g_signal_connect (notification,
+                                  "closed",
+                                  G_CALLBACK (on_notification_closed),
+                                  NULL);
+
+                notify_notification_set_hint (notification, "transient", g_variant_new_boolean (TRUE));
+                notify_notification_set_urgency (notification, NOTIFY_URGENCY_CRITICAL);
+                notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
+                if (has_disk_analyzer) {
+                        notify_notification_add_action (notification,
+                                                        "examine",
+                                                        _("Examine"),
+                                                        (NotifyActionCallback) examine_callback,
+                                                        g_strdup (path),
+                                                        g_free);
+                }
+                if (has_trash) {
+                        notify_notification_add_action (notification,
+                                                        "empty-trash",
+                                                        _("Empty Trash"),
+                                                        (NotifyActionCallback) empty_trash_callback,
+                                                        NULL,
+                                                        NULL);
+                }
+                notify_notification_add_action (notification,
+                                                "ignore",
+                                                _("Ignore"),
+                                                (NotifyActionCallback) ignore_callback,
+                                                NULL,
+                                                NULL);
+                notify_notification_set_category (notification, "device");
+
+                if (!notify_notification_show (notification, NULL)) {
+                        g_warning ("failed to send disk space notification\n");
+                }
+
+        } else {
+                dialog = gsd_ldsm_dialog_new (other_usable_volumes,
+                                              multiple_volumes,
+                                              has_disk_analyzer,
+                                              has_trash,
+                                              free_space,
+                                              name,
+                                              path);
+
+                g_object_ref (G_OBJECT (dialog));
+                response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+                gtk_widget_destroy (GTK_WIDGET (dialog));
+                dialog = NULL;
+
+                switch (response) {
+                case GTK_RESPONSE_CANCEL:
+                        retval = FALSE;
+                        break;
+                case GSD_LDSM_DIALOG_RESPONSE_ANALYZE:
+                        retval = FALSE;
+                        ldsm_analyze_path (path);
+                        break;
+                case GSD_LDSM_DIALOG_RESPONSE_EMPTY_TRASH:
+                        retval = TRUE;
+                        gsd_ldsm_trash_empty ();
+                        break;
+                case GTK_RESPONSE_NONE:
+                case GTK_RESPONSE_DELETE_EVENT:
+                        retval = TRUE;
+                        break;
+                default:
+                        g_assert_not_reached ();
+                }
+        }
 
         g_free (name);
-
-        g_object_ref (G_OBJECT (dialog));
-        response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-        gtk_widget_destroy (GTK_WIDGET (dialog));
-        dialog = NULL;
-
-        switch (response) {
-        case GTK_RESPONSE_CANCEL:
-                retval = FALSE;
-                break;
-        case GSD_LDSM_DIALOG_RESPONSE_ANALYZE:
-                retval = FALSE;
-                ldsm_analyze_path (g_unix_mount_get_mount_path (mount->mount));
-                break;
-        case GSD_LDSM_DIALOG_RESPONSE_EMPTY_TRASH:
-                retval = TRUE;
-                gsd_ldsm_trash_empty ();
-                break;
-        case GTK_RESPONSE_NONE:
-        case GTK_RESPONSE_DELETE_EVENT:
-                retval = TRUE;
-                break;
-        default:
-                g_assert_not_reached ();
-        }
+        g_free (path);
 
         return retval;
 }
@@ -306,25 +443,41 @@ ldsm_mount_should_ignore (GUnixMountEntry *mount)
          * expose this in a way that allows it to be used for this
          * purpose
          */
+         
+         /* We also ignore network filesystems */
                  
         const gchar *ignore_fs[] = {
+                "adfs",
+                "afs",
                 "auto",
                 "autofs",
+                "autofs4",
+                "cifs",
+                "cxfs",
                 "devfs",
                 "devpts",
                 "ecryptfs",
+                "gfs",
+                "gfs2",
                 "kernfs",
                 "linprocfs",
+                "linsysfs",
+                "lustre",
+                "lustre_lite",
+                "ncpfs",
+                "nfs",
+                "nfs4",
+                "nfsd",
+                "ocfs2",
                 "proc",
                 "procfs",
                 "ptyfs",
+                "rpc_pipefs",
                 "selinuxfs",
-                "linsysfs",
+                "smbfs",
                 "sysfs",
                 "tmpfs",
                 "usbfs",
-                "nfsd",
-                "rpc_pipefs",
                 "zfs",
                 NULL
         };
@@ -568,70 +721,50 @@ ldsm_is_hash_item_in_ignore_paths (gpointer key,
 static void
 gsd_ldsm_get_config ()
 {
-        GError *error = NULL;
+        gchar **settings_list;
 
-        free_percent_notify = gconf_client_get_float (client,
-                                                      GCONF_HOUSEKEEPING_DIR "/" GCONF_FREE_PC_NOTIFY_KEY,
-                                                      &error);
-        if (error != NULL) {
-                g_warning ("Error reading configuration from GConf: %s", error->message ? error->message : "Unknown error");
-                g_clear_error (&error);
-        }
+        free_percent_notify = g_settings_get_double (settings, SETTINGS_FREE_PC_NOTIFY_KEY);
         if (free_percent_notify >= 1 || free_percent_notify < 0) {
                 g_warning ("Invalid configuration of free_percent_notify: %f\n" \
                            "Using sensible default", free_percent_notify);
                 free_percent_notify = 0.05;
         }
 
-        free_percent_notify_again = gconf_client_get_float (client,
-                                                            GCONF_HOUSEKEEPING_DIR "/" GCONF_FREE_PC_NOTIFY_AGAIN_KEY,
-                                                            &error);
-        if (error != NULL) {
-                g_warning ("Error reading configuration from GConf: %s", error->message ? error->message : "Unknown error");
-                g_clear_error (&error);
-        }
+        free_percent_notify_again = g_settings_get_double (settings, SETTINGS_FREE_PC_NOTIFY_AGAIN_KEY);
         if (free_percent_notify_again >= 1 || free_percent_notify_again < 0) {
                 g_warning ("Invalid configuration of free_percent_notify_again: %f\n" \
                            "Using sensible default\n", free_percent_notify_again);
                 free_percent_notify_again = 0.01;
         }
 
-        free_size_gb_no_notify = gconf_client_get_int (client,
-                                                       GCONF_HOUSEKEEPING_DIR "/" GCONF_FREE_SIZE_NO_NOTIFY,
-                                                       &error);
-        if (error != NULL) {
-                g_warning ("Error reading configuration from GConf: %s", error->message ? error->message : "Unknown error");
-                g_clear_error (&error);
-        }
-         min_notify_period = gconf_client_get_int (client,
-                                                   GCONF_HOUSEKEEPING_DIR "/" GCONF_MIN_NOTIFY_PERIOD,
-                                                   &error);
-         if (error != NULL) {
-                 g_warning ("Error reading configuration from GConf: %s", error->message ? error->message : "Unkown error");
-                 g_clear_error (&error);
-         }
+        free_size_gb_no_notify = g_settings_get_int (settings, SETTINGS_FREE_SIZE_NO_NOTIFY);
+        min_notify_period = g_settings_get_int (settings, SETTINGS_MIN_NOTIFY_PERIOD);
 
-         if (ignore_paths != NULL) {
+        if (ignore_paths != NULL) {
                 g_slist_foreach (ignore_paths, (GFunc) g_free, NULL);
                 g_slist_free (ignore_paths);
-         }
-         ignore_paths = gconf_client_get_list (client,
-                                               GCONF_HOUSEKEEPING_DIR "/" GCONF_IGNORE_PATHS,
-                                               GCONF_VALUE_STRING, &error);
-         if (error != NULL) {
-                 g_warning ("Error reading configuration from GConf: %s", error->message ? error->message : "Unkown error");
-                 g_clear_error (&error);
-         } else {
+        }
+
+        settings_list = g_settings_get_strv (settings, SETTINGS_IGNORE_PATHS);
+        if (settings_list != NULL) {
+                gint i;
+
+                for (i = 0; i < G_N_ELEMENTS (settings_list); i++) {
+                        if (settings_list[i] != NULL)
+                                ignore_paths = g_slist_append (ignore_paths, settings_list[i]);
+                }
+
                 /* Make sure we dont leave stale entries in ldsm_notified_hash */
-                 g_hash_table_foreach_remove (ldsm_notified_hash,
-                                              ldsm_is_hash_item_in_ignore_paths, NULL);
-         }
+                g_hash_table_foreach_remove (ldsm_notified_hash,
+                                             ldsm_is_hash_item_in_ignore_paths, NULL);
+
+                g_strfreev (settings_list);
+        }
 }
 
 static void
-gsd_ldsm_update_config (GConfClient *client,
-                        guint cnxn_id,
-                        GConfEntry *entry,
+gsd_ldsm_update_config (GSettings *settings,
+                        const gchar *key,
                         gpointer user_data)
 {
         gsd_ldsm_get_config ();
@@ -640,8 +773,6 @@ gsd_ldsm_update_config (GConfClient *client,
 void
 gsd_ldsm_setup (gboolean check_now)
 {
-        GError          *error = NULL;
-
         if (ldsm_notified_hash || ldsm_timeout_id || ldsm_monitor) {
                 g_warning ("Low disk space monitor already initialized.");
                 return;
@@ -651,20 +782,10 @@ gsd_ldsm_setup (gboolean check_now)
                                                     g_free,
                                                     ldsm_free_mount_info);
 
-        client = gconf_client_get_default ();
-        if (client != NULL) {
-                gsd_ldsm_get_config ();
-                gconf_notify_id = gconf_client_notify_add (client,
-                                                           GCONF_HOUSEKEEPING_DIR,
-                                                           (GConfClientNotifyFunc) gsd_ldsm_update_config,
-                                                           NULL, NULL, &error);
-                if (error != NULL) {
-                        g_warning ("Cannot register callback for GConf notification");
-                        g_clear_error (&error);
-                }
-        } else {
-                g_warning ("Failed to get default client");
-        }
+        settings = g_settings_new (SETTINGS_HOUSEKEEPING_DIR);
+        gsd_ldsm_get_config ();
+        g_signal_connect (G_OBJECT (settings), "changed",
+                          G_CALLBACK (gsd_ldsm_update_config), NULL);
 
         ldsm_monitor = g_unix_mount_monitor_new ();
         g_unix_mount_monitor_set_rate_limit (ldsm_monitor, 1000);
@@ -694,9 +815,8 @@ gsd_ldsm_clean (void)
                 g_object_unref (ldsm_monitor);
         ldsm_monitor = NULL;
 
-        if (client) {
-                gconf_client_notify_remove (client, gconf_notify_id);
-                g_object_unref (client);
+        if (settings != NULL) {
+                g_object_unref (settings);
         }
 
         if (dialog) {
@@ -704,30 +824,14 @@ gsd_ldsm_clean (void)
                 dialog = NULL;
         }
 
+        if (notification != NULL) {
+                notify_notification_close (notification, NULL);
+                notification = NULL;
+        }
+
         if (ignore_paths) {
                 g_slist_foreach (ignore_paths, (GFunc) g_free, NULL);
                 g_slist_free (ignore_paths);
         }
 }
 
-#ifdef TEST
-int
-main (int    argc,
-      char **argv)
-{
-        GMainLoop *loop;
-
-        gtk_init (&argc, &argv);
-
-        loop = g_main_loop_new (NULL, FALSE);
-
-        gsd_ldsm_setup (TRUE);
-
-        g_main_loop_run (loop);
-
-        gsd_ldsm_clean ();
-        g_main_loop_unref (loop);
-
-        return 0;
-}
-#endif /* TEST */
