@@ -1759,7 +1759,7 @@ gsd_xrandr_manager_start (GsdXrandrManager *manager,
         log_msg ("------------------------------------------------------------\nSTARTING XRANDR PLUGIN\n");
 
         manager->priv->rw_screen = gnome_rr_screen_new (gdk_screen_get_default (), error);
-        g_signal_connect (manager->priv->rw_screen, "screen-changed", G_CALLBACK (on_randr_event), manager);
+        g_signal_connect (manager->priv->rw_screen, "changed", G_CALLBACK (on_randr_event), manager);
 
         if (manager->priv->rw_screen == NULL) {
                 log_msg ("Could not initialize the RANDR plugin%s%s\n",
